#ifndef AUDIO_AUDIO_H
#define AUDIO_AUDIO_H

/*
**    audio.h
**
**    (C) Copyright 2002-2003 Rocklyte Systems.
**        All Rights Reserved.
**
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Audio class.
*/

#define VER_AUDIO ((FLOAT)1.0)

#define MIN_STREAMBUFFER 65536

#define ADF_OVERSAMPLING   0x00000001   /* Enables oversampling for higher quality */
#define ADF_MUTE           0x00000002   /* Enables muting */
#define ADF_FILTERLOW      0x00000004   /* Enable a low level of output filtering */
#define ADF_FILTERHIGH     0x00000008   /* Enable a high level of output filtering */
#define ADF_STEREO         0x00000010   /* Enable stereo output */
#define ADF_16BIT          0x00000020   /* Enable 16 bit output */
#define ADF_VOLRAMPING     0x00000040   /* Enable volume ramping for softer playback */

/*****************************************************************************
** Channel flags
*/

#define CHF_MUTE       0x00000001   /* Channel is muted */
#define CHF_BACKWARD   0x00000002   /* Play channel backwards */
#define CHF_VOLRAMP    0x00000004   /* Volume ramping is enabled */
#define CHF_CHANGED    0x00000008   /* Sample change */

/*****************************************************************************
** Sample Formats
*/

#define SFM_U8BITMONO    1      /* 8-bit mono unsigned sample */
#define SFM_S16BITMONO   2      /* 16-bit mono signed sample, little-endian */
#define SFM_U8BITSTEREO  3      /* 8-bit stereo unsigned sample */
#define SFM_S16BITSTEREO 4      /* 16-bit stereo signed sample, little-endian */
#define SFM_END          5

/*** The following formats are supported by AddSample and AddStream for conversion purposes ***/
#define SFM_S8BITMONO    5      /* 8-bit mono signed sample */
#define SFM_U16BITMONO   6      /* 16-bit mono unsigned sample, little-endian */
#define SFM_S8BITSTEREO  7      /* 8-bit stereo signed sample */
#define SFM_U16BITSTEREO 8      /* 16-bit stereo unsigned sample, little-endian */

#define SFM_BIGENDIAN 0x80000000 /* Big endian flag */

/*****************************************************************************
** Loop settings.
*/

struct AudioLoop {
   WORD LoopMode;       /* Loop mode (single, double) */
   BYTE Loop1Type;      /* First loop type (unidirectional, bidirectional) */
   BYTE Loop2Type;      /* Second loop type (unidirectional, bidirectional) */
   LONG Loop1Start;     /* Start of the first loop */
   LONG Loop1End;       /* End of the first loop */
   LONG Loop2Start;     /* Start of the second loop */
   LONG Loop2End;       /* End of the second loop */
};

#define LOOP_SINGLE         1  /* Single loop: Releasing will end the note */
#define LOOP_SINGLE_RELEASE 2  /* Single loop: Sample data after the loop will be played when the note is released */
#define LOOP_DOUBLE         3  /* Double loop: When the note is released, playing shifts to the second loop */
#define LOOP_AMIGA_NONE     4  /* Amiga loop: Do nothing */
#define LOOP_AMIGA          5  /* Single loop: Amiga style */

#define LTYPE_UNIDIRECTIONAL 1
#define LTYPE_BIDIRECTIONAL  2

/*****************************************************************************
** Audio methods
*/

#define MT_OpenChannels   -1
#define MT_CloseChannels   -2
#define MT_AddSample      -3
#define MT_RemoveSample   -4
#define MT_BufferCommand  -5
#define MT_AddStream      -6
#define MT_Beep           -7

struct mtAddSample {
   LONG SampleFormat;       /* 8 bit, 16 bit, mono, stereo... */
   APTR Data;               /* Sample data */
   LONG DataSize;           /* Size of sample data, in bytes */
   struct AudioLoop *Loop;  /* Loop information (optional) */
   LONG LoopSize;           /* Size of loop information structure, in bytes */
   LONG Result;             /* Sample handle returned here */
};

struct mtAddStream {
   STRING Location;         /* The file that contains the sample data */
   OBJECTID ObjectID;       /* OR: The public object that contains the sample data */
   LONG SeekStart;          /* Offset to use when seeking to the start of sample data */
   LONG SampleFormat;       /* 8 bit, 16 bit, mono, stereo... */
   LONG SampleLength;       /* Total length of the sample to be streamed */
   LONG BufferLength;       /* Byte-length of the internal audio stream buffer */
   struct AudioLoop *Loop;  /* Loop information (optional) */
   LONG LoopSize;           /* Size of loop information structure, in bytes */
   LONG Result;             /* Sample handle returned here */
};

struct mtBeep {
   LONG Pitch;
   LONG Volume;
   LONG Duration;
};

struct mtBufferCommand {
   LONG CommandID;          /* Command ID */
   LONG Handle;             /* Channel handle or specific channel number */
   LONG Data;               /* Special command data */
};

struct mtCloseChannels {
   LONG Handle;
};

struct mtOpenChannels {
   LONG Amount;            /* Total number of channels to allocate */
   LONG Key;               /* Special key marker */
   LONG Commands;          /* Number of command buffers to allocate (approximate) */
   LONG Result;
};

struct mtRemoveSample {
   LONG Handle;
};

/*****************************************************************************
** Audio channel commands.
*/

#define CMD_StartSequence 1   /* New set of commands start */
#define CMD_EndSequence   2   /* Terminate current command set */
#define CMD_SetSample     3   /* ARG: Sample - Set a new sample on a channel */
#define CMD_SetVolume     4   /* ARG: Volume */
#define CMD_SetPan        5   /* ARG: Pan */
#define CMD_SetFrequency  6   /* ARG: Frequency */
#define CMD_SetRate       7   /* ARG: MilliSeconds */
#define CMD_Stop          8   /* Stop a channel's audio playback */
#define CMD_StopLooping   9   /* Stop a channel's audio playback with respect to looping */
#define CMD_SetPosition   10  /* ARG: Position */
#define CMD_Play          11  /* ARG: Frequency */
#define CMD_FadeIn        12  /* Fade in a channel */
#define CMD_FadeOut       13  /* Fade out a channel */
#define CMD_Mute          14  /* ARG: Mute */
#define CMD_SetLength     15  /* ARG: ByteLength */
#define CMD_Continue      16  /* Continue a sound sample that has been stopped */

/*****************************************************************************
** Helper functions.
*/

static ERROR CloseChannels(OBJECTID AudioID, LONG Handle)
{
   OBJECTPTR audio;
   struct mtCloseChannels close;
   extern struct KernelBase *KernelBase;

   if (AccessObject(AudioID, 5000, &audio) IS ERR_Okay) {
      close.Handle = Handle;
      Action(MT_CloseChannels, audio, &close);
      ReleaseObject(audio);
      return(ERR_Okay);
   }
   else return(ERR_AccessObject);
}

static ERROR OpenChannels(OBJECTID AudioID, LONG Amount, LONG Key, LONG Commands, LONG *Handle)
{
   OBJECTPTR audio;
   struct mtOpenChannels open;
   extern struct KernelBase *KernelBase;

   if (AccessObject(AudioID, 5000, &audio) IS ERR_Okay) {
      open.Amount   = Amount;
      open.Key      = Key;
      open.Commands = Commands;
      Action(MT_OpenChannels, audio, &open);
      *Handle = open.Result;
      ReleaseObject(audio);
      return(ERR_Okay);
   }
   else return(ERR_AccessObject);
}

#endif /* AUDIO_AUDIO_H */
